# pyright: reportConstantRedefinition=none
"""The firmware commands module.

These commands are used in the following models:
MP5103

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - firmware.load()
    - firmware.update()
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Firmware(BaseTSPCmd):
    """The ``firmware`` command tree.

    Properties and methods:
        - ``.load()``: The ``firmware.load()`` function.
        - ``.update()``: The ``firmware.update()`` function.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "firmware") -> None:
        super().__init__(device, cmd_syntax)

    def load(self, filename: str) -> None:
        """Run the ``firmware.load()`` function.

        Description:
            - This function loads a firmware image file from a USB flash drive.

        TSP Syntax:
            ```
            - firmware.load()
            ```

        Args:
            filename: Name of the firmware image file to load.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.load({filename})"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.load()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def update(self) -> None:
        """Run the ``firmware.update()`` function.

        Description:
            - This function updates the mainframe with a firmware image.

        TSP Syntax:
            ```
            - firmware.update()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.update()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.update()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The vertical commands module.

These commands are used in the following models:
MSO2, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - VERTical:DESKew:FROM:CUSTOMPROPAgation <NR3>
    - VERTical:DESKew:FROM:CUSTOMPROPAgation?
    - VERTical:DESKew:FROM:SOUrce CH<x>
    - VERTical:DESKew:FROM:SOUrce?
    - VERTical:DESKew:STATIC EXECute
    - VERTical:DESKew:TO:CUSTOMPROPAgation <NR3>
    - VERTical:DESKew:TO:SOUrce CH<x>
    - VERTical:DESKew:TO:SOUrce?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class VerticalDeskewToSource(SCPICmdWrite, SCPICmdRead):
    """The ``VERTical:DESKew:TO:SOUrce`` command.

    Description:
        - This command sets or queries the target channel for performing channel-to-channel deskew
          adjustment. Target sources can be any of the live analog channels.

    Usage:
        - Using the ``.query()`` method will send the ``VERTical:DESKew:TO:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``VERTical:DESKew:TO:SOUrce?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VERTical:DESKew:TO:SOUrce value``
          command.

    SCPI Syntax:
        ```
        - VERTical:DESKew:TO:SOUrce CH<x>
        - VERTical:DESKew:TO:SOUrce?
        ```

    Info:
        - ``CH<x>`` specifies an analog channel as the source.
    """


class VerticalDeskewToCustompropagation(SCPICmdWrite):
    """The ``VERTical:DESKew:TO:CUSTOMPROPAgation`` command.

    Description:
        - This command sets or queries a target (TO) delay that can be specified by the user when
          the propagation delay of the target (TO) probe used for deskew cannot be detected
          automatically.

    Usage:
        - Using the ``.write(value)`` method will send the
          ``VERTical:DESKew:TO:CUSTOMPROPAgation value`` command.

    SCPI Syntax:
        ```
        - VERTical:DESKew:TO:CUSTOMPROPAgation <NR3>
        ```

    Info:
        - ``<NR3>`` is a target (TO) delay that can be specified by the user when the propagation
          delay of the target (TO) probe used for deskew cannot be detected automatically.
    """


class VerticalDeskewTo(SCPICmdRead):
    """The ``VERTical:DESKew:TO`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``VERTical:DESKew:TO?`` query.
        - Using the ``.verify(value)`` method will send the ``VERTical:DESKew:TO?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.custompropagation``: The ``VERTical:DESKew:TO:CUSTOMPROPAgation`` command.
        - ``.source``: The ``VERTical:DESKew:TO:SOUrce`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._custompropagation = VerticalDeskewToCustompropagation(
            device, f"{self._cmd_syntax}:CUSTOMPROPAgation"
        )
        self._source = VerticalDeskewToSource(device, f"{self._cmd_syntax}:SOUrce")

    @property
    def custompropagation(self) -> VerticalDeskewToCustompropagation:
        """Return the ``VERTical:DESKew:TO:CUSTOMPROPAgation`` command.

        Description:
            - This command sets or queries a target (TO) delay that can be specified by the user
              when the propagation delay of the target (TO) probe used for deskew cannot be detected
              automatically.

        Usage:
            - Using the ``.write(value)`` method will send the
              ``VERTical:DESKew:TO:CUSTOMPROPAgation value`` command.

        SCPI Syntax:
            ```
            - VERTical:DESKew:TO:CUSTOMPROPAgation <NR3>
            ```

        Info:
            - ``<NR3>`` is a target (TO) delay that can be specified by the user when the
              propagation delay of the target (TO) probe used for deskew cannot be detected
              automatically.
        """
        return self._custompropagation

    @property
    def source(self) -> VerticalDeskewToSource:
        """Return the ``VERTical:DESKew:TO:SOUrce`` command.

        Description:
            - This command sets or queries the target channel for performing channel-to-channel
              deskew adjustment. Target sources can be any of the live analog channels.

        Usage:
            - Using the ``.query()`` method will send the ``VERTical:DESKew:TO:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``VERTical:DESKew:TO:SOUrce?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VERTical:DESKew:TO:SOUrce value``
              command.

        SCPI Syntax:
            ```
            - VERTical:DESKew:TO:SOUrce CH<x>
            - VERTical:DESKew:TO:SOUrce?
            ```

        Info:
            - ``CH<x>`` specifies an analog channel as the source.
        """
        return self._source


class VerticalDeskewStatic(SCPICmdWrite):
    """The ``VERTical:DESKew:STATIC`` command.

    Description:
        - This command executes static deskew using the deskew settings.

    Usage:
        - Using the ``.write(value)`` method will send the ``VERTical:DESKew:STATIC value`` command.

    SCPI Syntax:
        ```
        - VERTical:DESKew:STATIC EXECute
        ```

    Info:
        - ``EXECute`` will execute static deskew using the deskew settings.
    """


class VerticalDeskewFromSource(SCPICmdWrite, SCPICmdRead):
    """The ``VERTical:DESKew:FROM:SOUrce`` command.

    Description:
        - This command sets or queries the source channel for performing channel-to-channel deskew
          adjustment. Sources can be any of the analog channels.

    Usage:
        - Using the ``.query()`` method will send the ``VERTical:DESKew:FROM:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``VERTical:DESKew:FROM:SOUrce?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VERTical:DESKew:FROM:SOUrce value``
          command.

    SCPI Syntax:
        ```
        - VERTical:DESKew:FROM:SOUrce CH<x>
        - VERTical:DESKew:FROM:SOUrce?
        ```

    Info:
        - ``CH<x>`` specifies an analog channel as the source.
    """


class VerticalDeskewFromCustompropagation(SCPICmdWrite, SCPICmdRead):
    """The ``VERTical:DESKew:FROM:CUSTOMPROPAgation`` command.

    Description:
        - This command sets or queries a target (FROM) delay that you can specify when the
          propagation delay of the target (FROM) probe used for deskew cannot be detected
          automatically.

    Usage:
        - Using the ``.query()`` method will send the ``VERTical:DESKew:FROM:CUSTOMPROPAgation?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``VERTical:DESKew:FROM:CUSTOMPROPAgation?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``VERTical:DESKew:FROM:CUSTOMPROPAgation value`` command.

    SCPI Syntax:
        ```
        - VERTical:DESKew:FROM:CUSTOMPROPAgation <NR3>
        - VERTical:DESKew:FROM:CUSTOMPROPAgation?
        ```

    Info:
        - ``<NR3>`` is a target (FROM) delay that you can specify when the propagation delay of the
          target (FROM) probe used for deskew cannot be detected automatically.
    """


class VerticalDeskewFrom(SCPICmdRead):
    """The ``VERTical:DESKew:FROM`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``VERTical:DESKew:FROM?`` query.
        - Using the ``.verify(value)`` method will send the ``VERTical:DESKew:FROM?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.custompropagation``: The ``VERTical:DESKew:FROM:CUSTOMPROPAgation`` command.
        - ``.source``: The ``VERTical:DESKew:FROM:SOUrce`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._custompropagation = VerticalDeskewFromCustompropagation(
            device, f"{self._cmd_syntax}:CUSTOMPROPAgation"
        )
        self._source = VerticalDeskewFromSource(device, f"{self._cmd_syntax}:SOUrce")

    @property
    def custompropagation(self) -> VerticalDeskewFromCustompropagation:
        """Return the ``VERTical:DESKew:FROM:CUSTOMPROPAgation`` command.

        Description:
            - This command sets or queries a target (FROM) delay that you can specify when the
              propagation delay of the target (FROM) probe used for deskew cannot be detected
              automatically.

        Usage:
            - Using the ``.query()`` method will send the
              ``VERTical:DESKew:FROM:CUSTOMPROPAgation?`` query.
            - Using the ``.verify(value)`` method will send the
              ``VERTical:DESKew:FROM:CUSTOMPROPAgation?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``VERTical:DESKew:FROM:CUSTOMPROPAgation value`` command.

        SCPI Syntax:
            ```
            - VERTical:DESKew:FROM:CUSTOMPROPAgation <NR3>
            - VERTical:DESKew:FROM:CUSTOMPROPAgation?
            ```

        Info:
            - ``<NR3>`` is a target (FROM) delay that you can specify when the propagation delay of
              the target (FROM) probe used for deskew cannot be detected automatically.
        """
        return self._custompropagation

    @property
    def source(self) -> VerticalDeskewFromSource:
        """Return the ``VERTical:DESKew:FROM:SOUrce`` command.

        Description:
            - This command sets or queries the source channel for performing channel-to-channel
              deskew adjustment. Sources can be any of the analog channels.

        Usage:
            - Using the ``.query()`` method will send the ``VERTical:DESKew:FROM:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``VERTical:DESKew:FROM:SOUrce?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VERTical:DESKew:FROM:SOUrce value``
              command.

        SCPI Syntax:
            ```
            - VERTical:DESKew:FROM:SOUrce CH<x>
            - VERTical:DESKew:FROM:SOUrce?
            ```

        Info:
            - ``CH<x>`` specifies an analog channel as the source.
        """
        return self._source


class VerticalDeskew(SCPICmdRead):
    """The ``VERTical:DESKew`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``VERTical:DESKew?`` query.
        - Using the ``.verify(value)`` method will send the ``VERTical:DESKew?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.from``: The ``VERTical:DESKew:FROM`` command tree.
        - ``.static``: The ``VERTical:DESKew:STATIC`` command.
        - ``.to``: The ``VERTical:DESKew:TO`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._from = VerticalDeskewFrom(device, f"{self._cmd_syntax}:FROM")
        self._static = VerticalDeskewStatic(device, f"{self._cmd_syntax}:STATIC")
        self._to = VerticalDeskewTo(device, f"{self._cmd_syntax}:TO")

    @property
    def from_(self) -> VerticalDeskewFrom:
        """Return the ``VERTical:DESKew:FROM`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``VERTical:DESKew:FROM?`` query.
            - Using the ``.verify(value)`` method will send the ``VERTical:DESKew:FROM?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.custompropagation``: The ``VERTical:DESKew:FROM:CUSTOMPROPAgation`` command.
            - ``.source``: The ``VERTical:DESKew:FROM:SOUrce`` command.
        """
        return self._from

    @property
    def static(self) -> VerticalDeskewStatic:
        """Return the ``VERTical:DESKew:STATIC`` command.

        Description:
            - This command executes static deskew using the deskew settings.

        Usage:
            - Using the ``.write(value)`` method will send the ``VERTical:DESKew:STATIC value``
              command.

        SCPI Syntax:
            ```
            - VERTical:DESKew:STATIC EXECute
            ```

        Info:
            - ``EXECute`` will execute static deskew using the deskew settings.
        """
        return self._static

    @property
    def to(self) -> VerticalDeskewTo:
        """Return the ``VERTical:DESKew:TO`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``VERTical:DESKew:TO?`` query.
            - Using the ``.verify(value)`` method will send the ``VERTical:DESKew:TO?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.custompropagation``: The ``VERTical:DESKew:TO:CUSTOMPROPAgation`` command.
            - ``.source``: The ``VERTical:DESKew:TO:SOUrce`` command.
        """
        return self._to


class Vertical(SCPICmdRead):
    """The ``VERTical`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``VERTical?`` query.
        - Using the ``.verify(value)`` method will send the ``VERTical?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.deskew``: The ``VERTical:DESKew`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "VERTical") -> None:
        super().__init__(device, cmd_syntax)
        self._deskew = VerticalDeskew(device, f"{self._cmd_syntax}:DESKew")

    @property
    def deskew(self) -> VerticalDeskew:
        """Return the ``VERTical:DESKew`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``VERTical:DESKew?`` query.
            - Using the ``.verify(value)`` method will send the ``VERTical:DESKew?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.from``: The ``VERTical:DESKew:FROM`` command tree.
            - ``.static``: The ``VERTical:DESKew:STATIC`` command.
            - ``.to``: The ``VERTical:DESKew:TO`` command tree.
        """
        return self._deskew



"""The save commands module.

These commands are used in the following models:
MDO3

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SAVe:ASSIgn:TYPe {IMAGe|WAVEform|SETUp|UNASsigned}
    - SAVe:ASSIgn:TYPe?
    - SAVe:EVENTtable:B<x> <file path>
    - SAVe:EVENTtable:BUS<x> <file path>
    - SAVe:IMAGe <file path>
    - SAVe:IMAGe:FILEFormat {PNG|BMP|TIFf}
    - SAVe:IMAGe:FILEFormat?
    - SAVe:IMAGe:INKSaver {ON|OFF|1|0}
    - SAVe:IMAGe:INKSaver?
    - SAVe:IMAGe:LAYout {LANdscape|PORTRait}
    - SAVe:IMAGe:LAYout?
    - SAVe:MASK <QString>
    - SAVe:SETUp {<file path>|<NR1>}
    - SAVe:WAVEform <Source>,<Destination>
    - SAVe:WAVEform:FILEFormat {INTERNal|SPREADSheet}
    - SAVe:WAVEform:FILEFormat:RF_BB_IQ {TIQ|MATLAB}
    - SAVe:WAVEform:FILEFormat:RF_BB_IQ?
    - SAVe:WAVEform:FILEFormat?
    - SAVe:WAVEform:GATIng {NONe|CURSors|SCREEN}
    - SAVe:WAVEform:GATIng?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SaveWaveformGating(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform:GATIng`` command.

    Description:
        - Specifies whether save waveform operations should save the entire waveform (NONe) or a
          specified portion of the waveform.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:GATIng?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:GATIng?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:GATIng value`` command.

    SCPI Syntax:
        ```
        - SAVe:WAVEform:GATIng {NONe|CURSors|SCREEN}
        - SAVe:WAVEform:GATIng?
        ```

    Info:
        - ``CURSors`` turns on cursors and the gates are the waveform record points at the cursor
          positions.
        - ``NONe`` saves the entire waveform.
        - ``SCREEN`` , if zoom is on, the gates are the start and end waveform record points of the
          zoom (upper) graticule, otherwise the gates are the start and end waveform record points
          of the main graticule.
    """


class SaveWaveformFileformatRfBbIq(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform:FILEFormat:RF_BB_IQ`` command.

    Description:
        - This command specifies the file format for saving the RF baseband I & Q data. The default
          format is TIQ. See the description of the ``SAVE:WAVEFORM`` command for detailed
          information about these formats.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:FILEFormat:RF_BB_IQ?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:FILEFormat:RF_BB_IQ?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SAVe:WAVEform:FILEFormat:RF_BB_IQ value`` command.

    SCPI Syntax:
        ```
        - SAVe:WAVEform:FILEFormat:RF_BB_IQ {TIQ|MATLAB}
        - SAVe:WAVEform:FILEFormat:RF_BB_IQ?
        ```

    Info:
        - ``TIQ`` specifies to use the TIQ file format for saving the RF baseband I & Q data.
        - ``MATLAB`` specifies to use the MATLAB file format for saving the RF baseband I & Q data.
    """


class SaveWaveformFileformat(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform:FILEFormat`` command.

    Description:
        - This command specifies the file format to be used when saving waveforms - either an
          internal format, .ISF, or an external comma-delimited spreadsheet format, .CSV, that
          includes waveform header and timing information.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:FILEFormat?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:FILEFormat?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:FILEFormat value``
          command.

    SCPI Syntax:
        ```
        - SAVe:WAVEform:FILEFormat {INTERNal|SPREADSheet}
        - SAVe:WAVEform:FILEFormat?
        ```

    Info:
        - ``INTERNal`` specifies to save waveforms using an internal format. The file name should be
          specified with .ISF filename extension. These files can be recalled as reference
          waveforms.
        - ``SPREADSheet`` specifies that waveform data is saved in a format that contains comma
          delimited values. These waveform data files should be named using the .CSV filename
          extension. Saving waveforms in .CSV format enables spreadsheet programs to import the
          data.

    Properties:
        - ``.rf_bb_iq``: The ``SAVe:WAVEform:FILEFormat:RF_BB_IQ`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._rf_bb_iq = SaveWaveformFileformatRfBbIq(device, f"{self._cmd_syntax}:RF_BB_IQ")

    @property
    def rf_bb_iq(self) -> SaveWaveformFileformatRfBbIq:
        """Return the ``SAVe:WAVEform:FILEFormat:RF_BB_IQ`` command.

        Description:
            - This command specifies the file format for saving the RF baseband I & Q data. The
              default format is TIQ. See the description of the ``SAVE:WAVEFORM`` command for
              detailed information about these formats.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:FILEFormat:RF_BB_IQ?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``SAVe:WAVEform:FILEFormat:RF_BB_IQ?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SAVe:WAVEform:FILEFormat:RF_BB_IQ value`` command.

        SCPI Syntax:
            ```
            - SAVe:WAVEform:FILEFormat:RF_BB_IQ {TIQ|MATLAB}
            - SAVe:WAVEform:FILEFormat:RF_BB_IQ?
            ```

        Info:
            - ``TIQ`` specifies to use the TIQ file format for saving the RF baseband I & Q data.
            - ``MATLAB`` specifies to use the MATLAB file format for saving the RF baseband I & Q
              data.
        """
        return self._rf_bb_iq


class SaveWaveform(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform`` command.

    Description:
        - This command saves the specified waveform to the specified destination reference memory,
          or saves the specified waveform(s) to the specified destination file.

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform value`` command.

    SCPI Syntax:
        ```
        - SAVe:WAVEform <Source>,<Destination>
        ```

    Info:
        - ``CH<x>`` - x has a minimum of 1 and a maximum of 4.
        - ``D<x>`` - x has a minimum of 0 and a maximum of 15.
        - ``EMEM`` (instruments with the arbitrary waveform feature) - EMEM is the arbitrary
          waveform edit memory.
        - ``ARB<x>`` is one of the internal arbitrary waveform storage locations.
        - ``RF_NORMal, RF_AVErage, RF_MAXHold, RF_MINHold`` - These sources can be saved to REF1-4,
          an ISF or CSV file.
        - ``RF_BB_IQ`` - specifies to save baseband I & Q data.
        - ``ALL`` - Specifies all displayed waveforms subject to the restrictions indicated in the
          note below for saving ALL waveforms.
        - ``REF<x>`` - For saving a single scope waveform to one of the 4 internal reference
          waveform memory storage locations (see descriptions for source waveforms above for which
          waveforms can be saved to REF1-4).
        - ``EMEM`` - For instruments with the arbitrary waveform feature, scope waveforms (CH1-4,
          MATH, REF1-4, D0-D15) can be saved to arbitrary waveform edit memory.
        - ``ARB<x>`` - For instruments with the arbitrary waveform feature, EMEM can be saved to one
          of the 4 internal arbitrary waveform memorylocations.
        - ``<file path>`` - Specifies the destination file path to save the specified waveform(s).

    Properties:
        - ``.fileformat``: The ``SAVe:WAVEform:FILEFormat`` command.
        - ``.gating``: The ``SAVe:WAVEform:GATIng`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._fileformat = SaveWaveformFileformat(device, f"{self._cmd_syntax}:FILEFormat")
        self._gating = SaveWaveformGating(device, f"{self._cmd_syntax}:GATIng")

    @property
    def fileformat(self) -> SaveWaveformFileformat:
        """Return the ``SAVe:WAVEform:FILEFormat`` command.

        Description:
            - This command specifies the file format to be used when saving waveforms - either an
              internal format, .ISF, or an external comma-delimited spreadsheet format, .CSV, that
              includes waveform header and timing information.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:FILEFormat?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:FILEFormat?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:FILEFormat value``
              command.

        SCPI Syntax:
            ```
            - SAVe:WAVEform:FILEFormat {INTERNal|SPREADSheet}
            - SAVe:WAVEform:FILEFormat?
            ```

        Info:
            - ``INTERNal`` specifies to save waveforms using an internal format. The file name
              should be specified with .ISF filename extension. These files can be recalled as
              reference waveforms.
            - ``SPREADSheet`` specifies that waveform data is saved in a format that contains comma
              delimited values. These waveform data files should be named using the .CSV filename
              extension. Saving waveforms in .CSV format enables spreadsheet programs to import the
              data.

        Sub-properties:
            - ``.rf_bb_iq``: The ``SAVe:WAVEform:FILEFormat:RF_BB_IQ`` command.
        """
        return self._fileformat

    @property
    def gating(self) -> SaveWaveformGating:
        """Return the ``SAVe:WAVEform:GATIng`` command.

        Description:
            - Specifies whether save waveform operations should save the entire waveform (NONe) or a
              specified portion of the waveform.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:GATIng?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:GATIng?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:GATIng value``
              command.

        SCPI Syntax:
            ```
            - SAVe:WAVEform:GATIng {NONe|CURSors|SCREEN}
            - SAVe:WAVEform:GATIng?
            ```

        Info:
            - ``CURSors`` turns on cursors and the gates are the waveform record points at the
              cursor positions.
            - ``NONe`` saves the entire waveform.
            - ``SCREEN`` , if zoom is on, the gates are the start and end waveform record points of
              the zoom (upper) graticule, otherwise the gates are the start and end waveform record
              points of the main graticule.
        """
        return self._gating


class SaveSetup(SCPICmdWrite):
    """The ``SAVe:SETUp`` command.

    Description:
        - Stores the state of the oscilloscope to a specified memory location. You can later use the
          ``*RCL`` command to restore the oscilloscope to this saved state.

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:SETUp value`` command.

    SCPI Syntax:
        ```
        - SAVe:SETUp {<file path>|<NR1>}
        ```

    Info:
        - ``<file path>`` is the target location for storing the setup file. <file path> is a quoted
          string that defines the file name and path. Input the file path using the form
          ``<drive>:<dir>``/<filename>. <extension> and one or <dir>s are optional. If you do not
          specify them, the oscilloscope will store the file in the current working directory.
          <filename> stands for a filename. (Use of wildcard characters in filenames is not
          supported.) Filename extensions are not required but are highly recommended. For setups,
          use the extension '.SET'.
        - ``<NR1>`` specifies a location for saving the current front-panel setup. The front-panel
          setup value ranges from 1 to 10. Using an out-of-range value causes an execution error.
          Any settings that have been stored previously at this location will be overwritten.
    """


class SaveMask(SCPICmdWrite):
    """The ``SAVe:MASK`` command.

    Description:
        - This command saves the current mask definition to the file specified with a quoted string,
          into the current working directory. You can recall the mask from the file by using the
          command ``RECALL:MASK A`` series of examples showing how to use mask commands for typical
          tasks is included in an appendix.

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:MASK value`` command.

    SCPI Syntax:
        ```
        - SAVe:MASK <QString>
        ```

    Info:
        - ``QString`` is a quoted string that is the name of the mask definition being saved to the
          current working directory.
    """

    _WRAP_ARG_WITH_QUOTES = True


class SaveImageLayout(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:IMAGe:LAYout`` command.

    Description:
        - This command specifies the layout to use for saved screen images.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:IMAGe:LAYout?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:LAYout?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:LAYout value`` command.

    SCPI Syntax:
        ```
        - SAVe:IMAGe:LAYout {LANdscape|PORTRait}
        - SAVe:IMAGe:LAYout?
        ```

    Info:
        - ``LANdscape`` specifies that screen images are saved in landscape format.
        - ``PORTRait`` specifies that screen images are saved in portrait format.
    """


class SaveImageInksaver(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:IMAGe:INKSaver`` command.

    Description:
        - This command specifies the current ink saver setting for the ``SAVE:IMAGE`` command. If
          set to 'ON' or '1', images will be generated using the ink saver palette. If set to 'OFF'
          or '0', images will be generated using the standard palette.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:IMAGe:INKSaver?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:INKSaver?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:INKSaver value`` command.

    SCPI Syntax:
        ```
        - SAVe:IMAGe:INKSaver {ON|OFF|1|0}
        - SAVe:IMAGe:INKSaver?
        ```

    Info:
        - ``OFF`` or 0 generates images from the Inksaver palette.
        - ``ON`` or 1 generates images using the Standard palette.
    """


class SaveImageFileformat(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:IMAGe:FILEFormat`` command.

    Description:
        - This command specifies the file format to use for saving screen images.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:IMAGe:FILEFormat?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:FILEFormat?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:FILEFormat value`` command.

    SCPI Syntax:
        ```
        - SAVe:IMAGe:FILEFormat {PNG|BMP|TIFf}
        - SAVe:IMAGe:FILEFormat?
        ```

    Info:
        - ``PNG`` saves the file in Portable Network Graphics format.
        - ``BMP`` saves the file in Microsoft Windows bitmap format.
        - ``TIFf`` saves the file in Tagged Image File Format.
    """


class SaveImage(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:IMAGe`` command.

    Description:
        - Saves a capture of the screen image to the specified file. Supported image formats are
          PNG, Windows Bitmap, and TIFF. The file format is specified by the
          ``SAVE:IMAGE:FILEFORMAT`` command. The ``SAVE:IMAGE:INKSAVER`` command determines whether
          the data is saved in InkSaver mode.

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:IMAGe value`` command.

    SCPI Syntax:
        ```
        - SAVe:IMAGe <file path>
        ```

    Info:
        - ``<file path>`` is a filename, including path, where the image will be saved. If you do
          not specify a directory, the oscilloscope will store the file in the current working
          directory. File name extensions are not required but are highly recommended.

    Properties:
        - ``.fileformat``: The ``SAVe:IMAGe:FILEFormat`` command.
        - ``.inksaver``: The ``SAVe:IMAGe:INKSaver`` command.
        - ``.layout``: The ``SAVe:IMAGe:LAYout`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._fileformat = SaveImageFileformat(device, f"{self._cmd_syntax}:FILEFormat")
        self._inksaver = SaveImageInksaver(device, f"{self._cmd_syntax}:INKSaver")
        self._layout = SaveImageLayout(device, f"{self._cmd_syntax}:LAYout")

    @property
    def fileformat(self) -> SaveImageFileformat:
        """Return the ``SAVe:IMAGe:FILEFormat`` command.

        Description:
            - This command specifies the file format to use for saving screen images.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:IMAGe:FILEFormat?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:FILEFormat?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:FILEFormat value``
              command.

        SCPI Syntax:
            ```
            - SAVe:IMAGe:FILEFormat {PNG|BMP|TIFf}
            - SAVe:IMAGe:FILEFormat?
            ```

        Info:
            - ``PNG`` saves the file in Portable Network Graphics format.
            - ``BMP`` saves the file in Microsoft Windows bitmap format.
            - ``TIFf`` saves the file in Tagged Image File Format.
        """
        return self._fileformat

    @property
    def inksaver(self) -> SaveImageInksaver:
        """Return the ``SAVe:IMAGe:INKSaver`` command.

        Description:
            - This command specifies the current ink saver setting for the ``SAVE:IMAGE`` command.
              If set to 'ON' or '1', images will be generated using the ink saver palette. If set to
              'OFF' or '0', images will be generated using the standard palette.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:IMAGe:INKSaver?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:INKSaver?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:INKSaver value``
              command.

        SCPI Syntax:
            ```
            - SAVe:IMAGe:INKSaver {ON|OFF|1|0}
            - SAVe:IMAGe:INKSaver?
            ```

        Info:
            - ``OFF`` or 0 generates images from the Inksaver palette.
            - ``ON`` or 1 generates images using the Standard palette.
        """
        return self._inksaver

    @property
    def layout(self) -> SaveImageLayout:
        """Return the ``SAVe:IMAGe:LAYout`` command.

        Description:
            - This command specifies the layout to use for saved screen images.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:IMAGe:LAYout?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:LAYout?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:LAYout value`` command.

        SCPI Syntax:
            ```
            - SAVe:IMAGe:LAYout {LANdscape|PORTRait}
            - SAVe:IMAGe:LAYout?
            ```

        Info:
            - ``LANdscape`` specifies that screen images are saved in landscape format.
            - ``PORTRait`` specifies that screen images are saved in portrait format.
        """
        return self._layout


class SaveEventtableBusItem(ValidatedDynamicNumberCmd, SCPICmdWrite):
    """The ``SAVe:EVENTtable:BUS<x>`` command.

    Description:
        - Saves the data from bus<x> to a specified file and location; where x is the bus number
          (1-2).

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:EVENTtable:BUS<x> value`` command.

    SCPI Syntax:
        ```
        - SAVe:EVENTtable:BUS<x> <file path>
        ```

    Info:
        - ``<file path>`` is a quoted string that defines the file name and path location where the
          event table will be stored.
    """


class SaveEventtableBItem(ValidatedDynamicNumberCmd, SCPICmdWrite):
    """The ``SAVe:EVENTtable:B<x>`` command.

    Description:
        - Saves the data from bus<x> to a specified file and location; where x is the bus number
          (1-2).

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:EVENTtable:B<x> value`` command.

    SCPI Syntax:
        ```
        - SAVe:EVENTtable:B<x> <file path>
        ```

    Info:
        - ``<file path>`` is a quoted string that defines the file name and path location where the
          event table will be stored.
    """


class SaveEventtable(SCPICmdRead):
    """The ``SAVe:EVENTtable`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:EVENTtable?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:EVENTtable?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.bus``: The ``SAVe:EVENTtable:BUS<x>`` command.
        - ``.b``: The ``SAVe:EVENTtable:B<x>`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bus: Dict[int, SaveEventtableBusItem] = DefaultDictPassKeyToFactory(
            lambda x: SaveEventtableBusItem(device, f"{self._cmd_syntax}:BUS{x}")
        )
        self._b: Dict[int, SaveEventtableBItem] = DefaultDictPassKeyToFactory(
            lambda x: SaveEventtableBItem(device, f"{self._cmd_syntax}:B{x}")
        )

    @property
    def bus(self) -> Dict[int, SaveEventtableBusItem]:
        """Return the ``SAVe:EVENTtable:BUS<x>`` command.

        Description:
            - Saves the data from bus<x> to a specified file and location; where x is the bus number
              (1-2).

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:EVENTtable:BUS<x> value``
              command.

        SCPI Syntax:
            ```
            - SAVe:EVENTtable:BUS<x> <file path>
            ```

        Info:
            - ``<file path>`` is a quoted string that defines the file name and path location where
              the event table will be stored.
        """
        return self._bus

    @property
    def b(self) -> Dict[int, SaveEventtableBItem]:
        """Return the ``SAVe:EVENTtable:B<x>`` command.

        Description:
            - Saves the data from bus<x> to a specified file and location; where x is the bus number
              (1-2).

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:EVENTtable:B<x> value``
              command.

        SCPI Syntax:
            ```
            - SAVe:EVENTtable:B<x> <file path>
            ```

        Info:
            - ``<file path>`` is a quoted string that defines the file name and path location where
              the event table will be stored.
        """
        return self._b


class SaveAssignType(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:ASSIgn:TYPe`` command.

    Description:
        - This command specifies the assignment of the data to be saved when the front-panel Save
          button is pressed.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:ASSIgn:TYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn:TYPe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:ASSIgn:TYPe value`` command.

    SCPI Syntax:
        ```
        - SAVe:ASSIgn:TYPe {IMAGe|WAVEform|SETUp|UNASsigned}
        - SAVe:ASSIgn:TYPe?
        ```

    Info:
        - ``IMAGe`` assigns the Save button to save screen images.
        - ``WAVEform`` assigns the Save button to save waveforms.
        - ``SETUp`` assigns the Save button to save setups.
        - ``UNASsigned`` sets the Save button to unassigned.
    """


class SaveAssign(SCPICmdRead):
    """The ``SAVe:ASSIgn`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:ASSIgn?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.type``: The ``SAVe:ASSIgn:TYPe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._type = SaveAssignType(device, f"{self._cmd_syntax}:TYPe")

    @property
    def type(self) -> SaveAssignType:
        """Return the ``SAVe:ASSIgn:TYPe`` command.

        Description:
            - This command specifies the assignment of the data to be saved when the front-panel
              Save button is pressed.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:ASSIgn:TYPe?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn:TYPe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:ASSIgn:TYPe value`` command.

        SCPI Syntax:
            ```
            - SAVe:ASSIgn:TYPe {IMAGe|WAVEform|SETUp|UNASsigned}
            - SAVe:ASSIgn:TYPe?
            ```

        Info:
            - ``IMAGe`` assigns the Save button to save screen images.
            - ``WAVEform`` assigns the Save button to save waveforms.
            - ``SETUp`` assigns the Save button to save setups.
            - ``UNASsigned`` sets the Save button to unassigned.
        """
        return self._type


class Save(SCPICmdRead):
    """The ``SAVe`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.assign``: The ``SAVe:ASSIgn`` command tree.
        - ``.eventtable``: The ``SAVe:EVENTtable`` command tree.
        - ``.image``: The ``SAVe:IMAGe`` command.
        - ``.mask``: The ``SAVe:MASK`` command.
        - ``.setup``: The ``SAVe:SETUp`` command.
        - ``.waveform``: The ``SAVe:WAVEform`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "SAVe") -> None:
        super().__init__(device, cmd_syntax)
        self._assign = SaveAssign(device, f"{self._cmd_syntax}:ASSIgn")
        self._eventtable = SaveEventtable(device, f"{self._cmd_syntax}:EVENTtable")
        self._image = SaveImage(device, f"{self._cmd_syntax}:IMAGe")
        self._mask = SaveMask(device, f"{self._cmd_syntax}:MASK")
        self._setup = SaveSetup(device, f"{self._cmd_syntax}:SETUp")
        self._waveform = SaveWaveform(device, f"{self._cmd_syntax}:WAVEform")

    @property
    def assign(self) -> SaveAssign:
        """Return the ``SAVe:ASSIgn`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:ASSIgn?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.type``: The ``SAVe:ASSIgn:TYPe`` command.
        """
        return self._assign

    @property
    def eventtable(self) -> SaveEventtable:
        """Return the ``SAVe:EVENTtable`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:EVENTtable?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:EVENTtable?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.bus``: The ``SAVe:EVENTtable:BUS<x>`` command.
            - ``.b``: The ``SAVe:EVENTtable:B<x>`` command.
        """
        return self._eventtable

    @property
    def image(self) -> SaveImage:
        """Return the ``SAVe:IMAGe`` command.

        Description:
            - Saves a capture of the screen image to the specified file. Supported image formats are
              PNG, Windows Bitmap, and TIFF. The file format is specified by the
              ``SAVE:IMAGE:FILEFORMAT`` command. The ``SAVE:IMAGE:INKSAVER`` command determines
              whether the data is saved in InkSaver mode.

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:IMAGe value`` command.

        SCPI Syntax:
            ```
            - SAVe:IMAGe <file path>
            ```

        Info:
            - ``<file path>`` is a filename, including path, where the image will be saved. If you
              do not specify a directory, the oscilloscope will store the file in the current
              working directory. File name extensions are not required but are highly recommended.

        Sub-properties:
            - ``.fileformat``: The ``SAVe:IMAGe:FILEFormat`` command.
            - ``.inksaver``: The ``SAVe:IMAGe:INKSaver`` command.
            - ``.layout``: The ``SAVe:IMAGe:LAYout`` command.
        """
        return self._image

    @property
    def mask(self) -> SaveMask:
        """Return the ``SAVe:MASK`` command.

        Description:
            - This command saves the current mask definition to the file specified with a quoted
              string, into the current working directory. You can recall the mask from the file by
              using the command ``RECALL:MASK A`` series of examples showing how to use mask
              commands for typical tasks is included in an appendix.

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:MASK value`` command.

        SCPI Syntax:
            ```
            - SAVe:MASK <QString>
            ```

        Info:
            - ``QString`` is a quoted string that is the name of the mask definition being saved to
              the current working directory.
        """
        return self._mask

    @property
    def setup(self) -> SaveSetup:
        """Return the ``SAVe:SETUp`` command.

        Description:
            - Stores the state of the oscilloscope to a specified memory location. You can later use
              the ``*RCL`` command to restore the oscilloscope to this saved state.

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:SETUp value`` command.

        SCPI Syntax:
            ```
            - SAVe:SETUp {<file path>|<NR1>}
            ```

        Info:
            - ``<file path>`` is the target location for storing the setup file. <file path> is a
              quoted string that defines the file name and path. Input the file path using the form
              ``<drive>:<dir>``/<filename>. <extension> and one or <dir>s are optional. If you do
              not specify them, the oscilloscope will store the file in the current working
              directory. <filename> stands for a filename. (Use of wildcard characters in filenames
              is not supported.) Filename extensions are not required but are highly recommended.
              For setups, use the extension '.SET'.
            - ``<NR1>`` specifies a location for saving the current front-panel setup. The
              front-panel setup value ranges from 1 to 10. Using an out-of-range value causes an
              execution error. Any settings that have been stored previously at this location will
              be overwritten.
        """
        return self._setup

    @property
    def waveform(self) -> SaveWaveform:
        """Return the ``SAVe:WAVEform`` command.

        Description:
            - This command saves the specified waveform to the specified destination reference
              memory, or saves the specified waveform(s) to the specified destination file.

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform value`` command.

        SCPI Syntax:
            ```
            - SAVe:WAVEform <Source>,<Destination>
            ```

        Info:
            - ``CH<x>`` - x has a minimum of 1 and a maximum of 4.
            - ``D<x>`` - x has a minimum of 0 and a maximum of 15.
            - ``EMEM`` (instruments with the arbitrary waveform feature) - EMEM is the arbitrary
              waveform edit memory.
            - ``ARB<x>`` is one of the internal arbitrary waveform storage locations.
            - ``RF_NORMal, RF_AVErage, RF_MAXHold, RF_MINHold`` - These sources can be saved to
              REF1-4, an ISF or CSV file.
            - ``RF_BB_IQ`` - specifies to save baseband I & Q data.
            - ``ALL`` - Specifies all displayed waveforms subject to the restrictions indicated in
              the note below for saving ALL waveforms.
            - ``REF<x>`` - For saving a single scope waveform to one of the 4 internal reference
              waveform memory storage locations (see descriptions for source waveforms above for
              which waveforms can be saved to REF1-4).
            - ``EMEM`` - For instruments with the arbitrary waveform feature, scope waveforms
              (CH1-4, MATH, REF1-4, D0-D15) can be saved to arbitrary waveform edit memory.
            - ``ARB<x>`` - For instruments with the arbitrary waveform feature, EMEM can be saved to
              one of the 4 internal arbitrary waveform memorylocations.
            - ``<file path>`` - Specifies the destination file path to save the specified
              waveform(s).

        Sub-properties:
            - ``.fileformat``: The ``SAVe:WAVEform:FILEFormat`` command.
            - ``.gating``: The ``SAVe:WAVEform:GATIng`` command.
        """
        return self._waveform



"""The vxi commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO2, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - VXI:ENAble {ON|OFF|1|0}
    - VXI:ENAble?
    - VXI:PORT:HIGH <NR1>
    - VXI:PORT:HIGH?
    - VXI:PORT:LOW <NR1>
    - VXI:PORT:LOW?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class VxiPortLow(SCPICmdWrite, SCPICmdRead):
    """The ``VXI:PORT:LOW`` command.

    Description:
        - This command sets or queries the lower end of the port range for the VXI-11 server.

    Usage:
        - Using the ``.query()`` method will send the ``VXI:PORT:LOW?`` query.
        - Using the ``.verify(value)`` method will send the ``VXI:PORT:LOW?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VXI:PORT:LOW value`` command.

    SCPI Syntax:
        ```
        - VXI:PORT:LOW <NR1>
        - VXI:PORT:LOW?
        ```

    Info:
        - ``<NR1>`` is the low end of the TCPIP port range for the VXI-11 server connection. Some
          ports are restricted and cannot be used because they are used by other services. If a
          restricted port falls between the VXI-11 server low port and the VXI-11 server high port,
          then the error code 224 (Illegal parameter value) is posted to the event queue and the
          VXI-11 low port remains unchanged. If the VXI-11 low port number is set to a valid value
          above the VXI-11 high port number, the VXI-11 high port number is set to the value of the
          VXI-11 low port number plus one.
    """


class VxiPortHigh(SCPICmdWrite, SCPICmdRead):
    """The ``VXI:PORT:HIGH`` command.

    Description:
        - This command sets or queries the higher end of the port range for the VXI-11 server.

    Usage:
        - Using the ``.query()`` method will send the ``VXI:PORT:HIGH?`` query.
        - Using the ``.verify(value)`` method will send the ``VXI:PORT:HIGH?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VXI:PORT:HIGH value`` command.

    SCPI Syntax:
        ```
        - VXI:PORT:HIGH <NR1>
        - VXI:PORT:HIGH?
        ```

    Info:
        - ``<NR1>`` is the high end of the TCPIP port range for the VXI-11 server connection. Some
          ports are restricted and cannot be used because they are used by other services. If a
          restricted port falls between the VXI-11 server low port and the VXI-11 server high port,
          then the error code 224 (Illegal parameter value) is posted to the event queue and the
          VXI-11 high port remains unchanged. If the VXI-11 high port number is set to a valid value
          below the VXI-11 low port number, the VXI-11 low port number is set to the value of the
          VXI-11 high port number minus one.
    """


class VxiPort(SCPICmdRead):
    """The ``VXI:PORT`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``VXI:PORT?`` query.
        - Using the ``.verify(value)`` method will send the ``VXI:PORT?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.high``: The ``VXI:PORT:HIGH`` command.
        - ``.low``: The ``VXI:PORT:LOW`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._high = VxiPortHigh(device, f"{self._cmd_syntax}:HIGH")
        self._low = VxiPortLow(device, f"{self._cmd_syntax}:LOW")

    @property
    def high(self) -> VxiPortHigh:
        """Return the ``VXI:PORT:HIGH`` command.

        Description:
            - This command sets or queries the higher end of the port range for the VXI-11 server.

        Usage:
            - Using the ``.query()`` method will send the ``VXI:PORT:HIGH?`` query.
            - Using the ``.verify(value)`` method will send the ``VXI:PORT:HIGH?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VXI:PORT:HIGH value`` command.

        SCPI Syntax:
            ```
            - VXI:PORT:HIGH <NR1>
            - VXI:PORT:HIGH?
            ```

        Info:
            - ``<NR1>`` is the high end of the TCPIP port range for the VXI-11 server connection.
              Some ports are restricted and cannot be used because they are used by other services.
              If a restricted port falls between the VXI-11 server low port and the VXI-11 server
              high port, then the error code 224 (Illegal parameter value) is posted to the event
              queue and the VXI-11 high port remains unchanged. If the VXI-11 high port number is
              set to a valid value below the VXI-11 low port number, the VXI-11 low port number is
              set to the value of the VXI-11 high port number minus one.
        """
        return self._high

    @property
    def low(self) -> VxiPortLow:
        """Return the ``VXI:PORT:LOW`` command.

        Description:
            - This command sets or queries the lower end of the port range for the VXI-11 server.

        Usage:
            - Using the ``.query()`` method will send the ``VXI:PORT:LOW?`` query.
            - Using the ``.verify(value)`` method will send the ``VXI:PORT:LOW?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VXI:PORT:LOW value`` command.

        SCPI Syntax:
            ```
            - VXI:PORT:LOW <NR1>
            - VXI:PORT:LOW?
            ```

        Info:
            - ``<NR1>`` is the low end of the TCPIP port range for the VXI-11 server connection.
              Some ports are restricted and cannot be used because they are used by other services.
              If a restricted port falls between the VXI-11 server low port and the VXI-11 server
              high port, then the error code 224 (Illegal parameter value) is posted to the event
              queue and the VXI-11 low port remains unchanged. If the VXI-11 low port number is set
              to a valid value above the VXI-11 high port number, the VXI-11 high port number is set
              to the value of the VXI-11 low port number plus one.
        """
        return self._low


class VxiEnable(SCPICmdWrite, SCPICmdRead):
    """The ``VXI:ENAble`` command.

    Description:
        - This command sets or queries the state of the VXI-11 server, which is used for command and
          control over an Ethernet connection.

    Usage:
        - Using the ``.query()`` method will send the ``VXI:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``VXI:ENAble?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VXI:ENAble value`` command.

    SCPI Syntax:
        ```
        - VXI:ENAble {ON|OFF|1|0}
        - VXI:ENAble?
        ```

    Info:
        - ``ON`` enables the VXI-11 server.
        - ``OFF`` disables the VXI-11 server.
        - ``1`` enables the VXI-11 server.
        - ``0`` disables the VXI-11 server.
    """


class Vxi(SCPICmdRead):
    """The ``VXI`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``VXI?`` query.
        - Using the ``.verify(value)`` method will send the ``VXI?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.enable``: The ``VXI:ENAble`` command.
        - ``.port``: The ``VXI:PORT`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "VXI") -> None:
        super().__init__(device, cmd_syntax)
        self._enable = VxiEnable(device, f"{self._cmd_syntax}:ENAble")
        self._port = VxiPort(device, f"{self._cmd_syntax}:PORT")

    @property
    def enable(self) -> VxiEnable:
        """Return the ``VXI:ENAble`` command.

        Description:
            - This command sets or queries the state of the VXI-11 server, which is used for command
              and control over an Ethernet connection.

        Usage:
            - Using the ``.query()`` method will send the ``VXI:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``VXI:ENAble?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VXI:ENAble value`` command.

        SCPI Syntax:
            ```
            - VXI:ENAble {ON|OFF|1|0}
            - VXI:ENAble?
            ```

        Info:
            - ``ON`` enables the VXI-11 server.
            - ``OFF`` disables the VXI-11 server.
            - ``1`` enables the VXI-11 server.
            - ``0`` disables the VXI-11 server.
        """
        return self._enable

    @property
    def port(self) -> VxiPort:
        """Return the ``VXI:PORT`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``VXI:PORT?`` query.
            - Using the ``.verify(value)`` method will send the ``VXI:PORT?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.high``: The ``VXI:PORT:HIGH`` command.
            - ``.low``: The ``VXI:PORT:LOW`` command.
        """
        return self._port
